from django.db import models

# Create your models here.
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, AbstractUser
from django.db import models
from django.utils import timezone
from django.conf import settings
from .storages import UthoStorage

class CustomUserManager(BaseUserManager):
    def create_user(self, mobile_or_email, password=None, **extra_fields):
        if not mobile_or_email:
            raise ValueError('The mobile or email field must be set')
        user = self.model(mobile_or_email=mobile_or_email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, mobile_or_email, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        return self.create_user(mobile_or_email, password, **extra_fields)

class CustomUser(AbstractUser):
    username = models.CharField(max_length=255, unique=True, null=True, blank=True)
    mobile_or_email = models.CharField(unique=True, max_length=255)
    device_type = models.CharField(max_length=50, null=True, blank=True)
    device_id = models.CharField(max_length=255, null=True, blank=True)
    device_model = models.CharField(max_length=255, null=True, blank=True)
    latitude = models.CharField(max_length=100, null=True, blank=True)
    longitude = models.CharField(max_length=100, null=True, blank=True)
    device_ipaddress = models.CharField(max_length=45, null=True, blank=True)
    login_method = models.CharField(max_length=50, choices=[('1', 'Traditional'), ('google', 'Google'), ('apple', 'Apple')])
    user_status = models.CharField(max_length=50, choices=[('active', 'Active'), ('inactive', 'Inactive')], default='inactive')
    is_verified = models.BooleanField(default=False)
    mobile_number = models.CharField(max_length=15, null=True, blank=True, unique=True)
    is_mobile_verified = models.BooleanField(default=False)
    full_name = models.CharField(max_length=255, null=True, blank=True)
    fcm_token = models.CharField(max_length=255, null=True, blank=True)
    agora_token = models.TextField(null=True, blank=True)
    agora_uid = models.CharField(max_length=100, null=True, blank=True)
    agora_username = models.CharField(max_length=100, null=True, blank=True)


    
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)
    is_superuser = models.BooleanField(default=False)

    USERNAME_FIELD = 'mobile_or_email'
    REQUIRED_FIELDS = []

    objects = CustomUserManager()

    def __str__(self):
        return self.mobile_or_email

class OTPVerification(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='otp_verifications')
    otp = models.CharField(max_length=6)
    otp_expires_at = models.DateTimeField()
    is_verified = models.BooleanField(default=False)
    
    def save(self, *args, **kwargs):
        if not self.otp_expires_at:
            self.otp_expires_at = timezone.now() + timezone.timedelta(minutes=10)  # OTP expiry time
        super().save(*args, **kwargs)

    def __str__(self):
        return f"OTP for {self.user.mobile_or_email}"

class UserSession(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='sessions')
    session_token = models.CharField(max_length=255, unique=True)
    expires_at = models.DateTimeField()
    
    def save(self, *args, **kwargs):
        if not self.expires_at:
            self.expires_at = timezone.now() + timezone.timedelta(days=30)  # Default session expiry
        super().save(*args, **kwargs)

    def __str__(self):
        return f"Session for {self.user.mobile_or_email}"


class PasswordResetRequest(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    otp = models.CharField(max_length=6)
    reset_token = models.CharField(max_length=255)
    otp_expires_at = models.DateTimeField()

    def is_expired(self):
        return timezone.now() > self.otp_expires_at



class OnboardingImage(models.Model):
    title = models.CharField(max_length=255)
    short_description = models.TextField()
    image = models.ImageField(upload_to='onboarding-images/', storage=UthoStorage)  # Using UthoStorage

    def __str__(self):
        return self.title

class ArtiztNetworkAccNumber(models.Model):
    user = models.OneToOneField(CustomUser, on_delete=models.CASCADE, related_name='artizt_account')
    account_number = models.CharField(max_length=50, unique=True)

    def __str__(self):
        return f"{self.account_number} - {self.user}"
    
class State(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

class City(models.Model):
    name = models.CharField(max_length=100)
    state = models.ForeignKey(State, related_name='cities', on_delete=models.CASCADE)

    def __str__(self):
        return self.name

class StateShortNames(models.Model):
    name = models.CharField(max_length=255)
    code = models.CharField(max_length=5)

    def __str__(self):
        return self.name

class DeviceList(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name="devices")
    session_id = models.CharField(max_length=255, unique=True)
    device_name = models.CharField(max_length=255)
    device_ip = models.GenericIPAddressField()
    login_time = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=50, choices=[("active", "Active"), ("logged_out", "Logged Out")], default="active")

    def __str__(self):
        return f"{self.user} - {self.device_name} ({self.status})"

class DeletedAccount(models.Model):
    user = models.OneToOneField(CustomUser, on_delete=models.CASCADE, related_name='deleted_account')
    reason = models.TextField()
    deleted_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"Deleted Account - {self.user.mobile_or_email}"

class Country(models.Model):
    name = models.CharField(max_length=100, unique=True)

    def __str__(self):
        return self.name
    
class PrivacySettings(models.Model):
    user = models.OneToOneField(CustomUser, on_delete=models.CASCADE, related_name="privacy_settings")
    enable_notifications = models.BooleanField(default=True)
    account_notifications = models.BooleanField(default=True)
    promotions_email = models.BooleanField(default=True)
    promotions_sms = models.BooleanField(default=True)
    promotions_whatsapp = models.BooleanField(default=True)
    app_updates = models.BooleanField(default=True)

    def __str__(self):
        return f"{self.user.username} - Privacy Settings"